@@ -44,7 +44,8 @@ typedef enum {
 	GNOME_VFS_XFER_NEW_UNIQUE_DIRECTORY = 1 << 7,
 	GNOME_VFS_XFER_REMOVESOURCE = 1 << 8,
 	GNOME_VFS_XFER_USE_UNIQUE_NAMES = 1 << 9,
-	GNOME_VFS_XFER_LINK_ITEMS = 1 << 10
+	GNOME_VFS_XFER_LINK_ITEMS = 1 << 10,
+	GNOME_VFS_XFER_FOLLOW_LINKS_RECURSIVE = 1 << 11,
 } GnomeVFSXferOptions;
 
 /* Progress status, to be reported to the caller of the transfer operation.  */
@@ -70,8 +71,19 @@ typedef enum {
 	GNOME_VFS_XFER_OVERWRITE_MODE_SKIP = 3
 } GnomeVFSXferOverwriteMode;
 
-/* This defines the actions to perform before a file is being overwritten
-   (i.e., these are the answers that can be given to a replace query).  */
+/**
+ * GnomeVFSXferOverwriteAction:
+ * @GNOME_VFS_XFER_OVERWRITE_ACTION_ABORT: abort the transfer
+ * @GNOME_VFS_XFER_OVERWRITE_ACTION_REPLACE: replace the existing file
+ * @GNOME_VFS_XFER_OVERWRITE_ACTION_REPLACE_ALL: replace the existing file, and all future files
+ * without prompting the callback.
+ * @GNOME_VFS_XFER_OVERWRITE_ACTION_SKIP: don't copy over the existing file
+ * @GNOME_VFS_XFER_OVERWRITE_ACTION_SKIP_ALL: don't copy over the existing file, and all future
+ * files without prompting the callback.
+ * 
+ * This defines the actions to perform before a file is being overwritten
+ * (i.e., these are the answers that can be given to a replace query).  
+ **/
 typedef enum {
 	GNOME_VFS_XFER_OVERWRITE_ACTION_ABORT = 0,
 	GNOME_VFS_XFER_OVERWRITE_ACTION_REPLACE = 1,
